module Data.Aeson.ParseUtils
where

import Data.Aeson
import Data.Yaml as YAML
import Data.HashMap.Strict (HashMap)
import qualified Data.HashMap.Strict as HashMap
import Data.ByteString (ByteString)
import Data.Maybe (fromMaybe)

fromJSONWithDefaults :: (ToJSON a, FromJSON a) => a -> Value -> Result a
fromJSONWithDefaults defaults =
    fromJSON . (mergeValues $ toJSON defaults)

mergeValues :: Value -> Value -> Value
-- Objects are handled recursively
mergeValues (Object x) (Object y) =
    Object (HashMap.unionWith mergeValues x y)
-- Null keeps the original value
mergeValues x Null = x
-- Anything else -> second value takes precedence
mergeValues _ x = x

decodeWithDefaults :: (ToJSON a, FromJSON a) => a -> ByteString -> Maybe a
decodeWithDefaults defaults src =
    YAML.decode src >>= (resultToMaybe . fromJSONWithDefaults defaults)
    where
        resultToMaybe (Success x) = Just x
        resultToMaybe (Error e) = Nothing
    
